unit Modmastu;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Grids, Menus, ToolHelp, Printers,
  IniFiles, AboutBox;

type
  TForm1 = class(TForm)
    StringGrid1: TStringGrid;
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    SaveList1: TMenuItem;
    Exit1: TMenuItem;
    Module1: TMenuItem;
    All1: TMenuItem;
    Exe1: TMenuItem;
    Dll1: TMenuItem;
    VBX1: TMenuItem;
    DRV1: TMenuItem;
    FON1: TMenuItem;
    About1: TMenuItem;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure StringGrid1TopLeftChanged(Sender: TObject);
    procedure StringGrid1SelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure ModuleClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure StringGrid1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure StringGrid1DrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure SaveList1Click(Sender: TObject);
  private
    { Private declarations }
    FType: integer;
    FColm: integer;
    function  GetColm: integer;
    procedure SetColm(Value:integer);
    function  GetFType: integer;
    procedure SetFType(Value:integer);
    procedure ResetFormLocation;
    procedure AdjustLastColWidth;
    procedure SelectType;
    procedure SortGrid( const ColNum : integer );
    procedure DecrementModuleUsage;
    function  FormattedOutput(var n: integer): string;
  public
    { Public declarations }
    property Colm: integer read GetColm write SetColm default 0;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
const
  ModTypes : array[0..5] of string[3] = ('*.*','EXE','DLL','VBX','DRV','FON');
  fsTextName: array[TFontStyle] of string[12] = ('fsBold','fsItalic',
                                                 'fsUnderline','fsStrikeOut');
  fpTextName: array[TFontPitch] of string[12] = ('fpDefault','fpVariable','fpFixed');
type
  CfType = function(var SG: TStringGrid; x,y,z: integer): boolean;

function  TForm1.GetColm: integer;
begin
  Result := FColm;
end;

procedure TForm1.SetColm(Value:integer);
begin
  if Value<>FColm then
    FColm := Value;
end;

function  TForm1.GetFType: integer;
begin
  Result := FType;
end;

procedure TForm1.SetFType(Value:integer);
begin
  if Value<>FType then
    FType := Value;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
with StringGrid1 do
 begin
  { initialise column headings and set widths }
  Cells[0,0] := ' Module';
  ColWidths[0] := Canvas.TextWidth('X') * 9;      { 8 chars + 1 }

  Cells[1,0] := 'Usage';
  ColWidths[1] := Canvas.TextWidth('Usage');      { should be small }

  Cells[2,0] := 'Size   ';
  ColWidths[2] := Canvas.TextWidth('8') * 9;      { very big number }

  Cells[3,0] := 'Date         Time    ';
  ColWidths[3] := Canvas.TextWidth('X') * 15;     { should be big enough }

  Cells[4,0] := ' Path & Filename';
  ColWidths[4] := Canvas.TextWidth('X') * 40;{ it will be recalculated later }

  { fire up the analysis }
  RowCount := 90;   { a large number as a attempt to stop flicker of redraw }
  Row := 1;
  SetColm(0);
  SetFType(0);
  All1.Checked := True;
 end;
end;

procedure TForm1.FormResize(Sender: TObject);
var
  cw, tw: integer;
begin
  if ClientHeight  < 400 then
     ClientHeight := 400;
  cw := ClientWidth;
  if cw<400 then
    begin
      cw := 400;
      ClientWidth := cw;
    end;
  with Button2 do
    Left := (cw - Width) div 2;
  with Button5 do
    Left := (cw - Width) div 2;
  with Button3 do
    Left := cw - Width - 18;
  with Button6 do
    Left := cw - Width - 18;
  AdjustLastColWidth;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
const
  working : boolean = false;
begin
  if (ssAlt in Shift) then
   if (char(Key) in ['D','R','U','P','L','I','X']) then
     begin
       if not(working) then
         begin
           working := true;
           case lo(Key) of
             ord('D'): Button5Click(Sender);       { ALT_D }
             ord('R'): Button6Click(Sender);       { ALT_R }
             ord('U'): Button4Click(Sender);       { ALT_U }
             ord('P'): Button3Click(Sender);       { ALT_P }
             ord('L'): Button1Click(Sender);       { ALT_L }
             ord('I'): Button2Click(Sender);       { ALT_I }
             ord('X'): Exit1Click(Sender);         { ALT_X }
           end;
           working := false;
         end;
       key := 0;
     end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  ResetFormLocation;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  SelectType;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Sysini: TIniFile;
  TFPitch: TFontPitch;
  TFStyle: TFontStyle;
  FString: String;
begin
  try
    Sysini := TInifile.Create(ChangeFileExt(paramstr(0),'.INI'));
    try
     with Form1,Sysini do
      begin
        WriteInteger('TMWCFG','MFTop',Top);
        WriteInteger('TMWCFG','MFLeft',Left);
        WriteInteger('TMWCFG','MFWidth',Width);
        WriteInteger('TMWCFG','MFHeight',Height);
        case WindowState of
          wsMinimized:  WriteInteger('TMWCFG','WinState', 1);
          wsNormal:     WriteInteger('TMWCFG','WinState', 2);
          wsMaximized:  WriteInteger('TMWCFG','WinState', 3);
        end;

        WriteString ('TPRTCFG','TPRTFN',Printer.Canvas.Font.Name);
        WriteInteger('TPRTCFG','TPRTFH',abs(Printer.Canvas.Font.Height));

        FString := '';
        for TFStyle := fsBold to fsStrikeOut do
         if TFStyle in Printer.Canvas.Font.Style then
          Fstring := Fstring+fsTextName[TFStyle]+',';
        if FString<>'' then
          dec(FString[0]);
        FString := FString + '          ;'; { add comments of TFont style to choose from }
        for TFStyle := fsBold to fsStrikeOut do
          Fstring := Fstring+fsTextName[TFStyle]+',';
        dec(FString[0]);
        WriteString ('TPRTCFG','TPRTFS',FString);

        FString := fpTextName[Printer.Canvas.Font.Pitch]+' ;';
        for TFpitch := fpDefault to fpFixed do
          FString := FString + fpTextName[TFpitch]+',';
        dec(FString[0]);
        WriteString('TPRTCFG','TPRTFP',FString);
      end;
    finally
      Sysini.Free;
    end;
  finally
    Action := caFree;
  end;
end;

procedure TForm1.ResetFormLocation;
var
  TFPitch: TFontPitch;
  TFStyle: TFontStyle;
  LFStyle: TFontStyles;
  FString: String[60];
  Sysini: TInifile;
  l,t,w,h,n: integer;
begin
  try
    Sysini := TInifile.Create(ChangeFileExt(paramstr(0),'.INI'));
    with Form1 do
      begin
        l := left;
        t := top;
        h := height;
        w := width;
        case WindowState of
          wsMinimized: n := 1;
          wsNormal:    n := 2;
          wsMaximized: n := 3;
        end;
      end;

    with Sysini do
      begin
        t := ReadInteger('TMWCFG','MFTop',t);
        l := ReadInteger('TMWCFG','MFLeft',l);
        w := ReadInteger('TMWCFG','MFWidth',w);
        h := ReadInteger('TMWCFG','MFHeight',h);
        n := ReadInteger('TMWCFG','WinState',n);
      end;

    with Screen do   { Range check input values }
      begin
        if (h<50)or(h>Height) then
          begin h := Height-50; t := 25; end;
        if (w<50)or(w>Width) then
          begin w := Width-50;  l := 25; end;
        if (t<0)or(t>h) then    t := 25;
        if (l<0)or(l>w) then    l := 25;
      end;

    with Form1 do
      begin
        SetBounds(l,t,w,h);
        case n of
          1: WindowState := wsMinimized;
          2: WindowState := wsNormal;
          3: WindowState := wsMaximized;
        end;
      end;

    with Sysini, Printer.Canvas.Font do
      begin
        Name    :=  ReadString ('TPRTCFG','TPRTFN','Courier New');
        Height  := -(Printer.PageHeight div 60); { set default 60 lines per page }
        Height  := -ReadInteger('TPRTCFG','TPRTFH', abs(Printer.Canvas.Font.Height));

        FString := ReadString ('TPRTCFG','TPRTFS','');
        if pos(';',FString)>0 then { strip comments of TFont styles off end }
          FString := Copy(FString,1,pos(';',FString)-1);
        LFStyle := [];
        for TFStyle := fsBold to fsStrikeOut do
         if (pos(fsTextName[TFStyle],FString)>0) then
          LFStyle := LFstyle + [TFStyle];
        Style := LFStyle;

        Fstring := fpTextName[Pitch];
        FString := ReadString('TPRTCFG','TPRTFP',FString);
        if pos(';',FString)>0 then
          FString := Copy(FString,1,pos(';',FString)-1);
        for TFPitch := fpDefault to fpFixed do
         if (pos(fpTextName[TFPitch],FString)>0) then
          Pitch := TFPitch;
     end;
  finally
    Sysini.free;
  end;
end;

procedure TForm1.AdjustLastColWidth;
var
  c, i, tw, sw, cw: integer;
begin
 with StringGrid1 do
   begin
     cw := ClientWidth;
     sw := 0;
     for i := 0 to RowCount-1 do           { find the minimum display width }
      begin                                { of the last column of filenames }
       tw := Canvas.TextWidth(Cells[4,i]); { by checking the textwidth of the }
       if tw>sw then sw := tw;             { current Font being used. }
      end;
     i := ColCount-1;
     tw := sw;
     while (i>0) and (tw<cw) do           { add in the width of others columns}
       begin                           { until it is greather than Clientwidth}
         dec(i);
         tw := tw + ColWidths[i] + GridLineWidth;
       end;
     if tw>cw then                     { if we added in too much then subtract}
       begin
         tw := tw - ColWidths[i] - GridLineWidth; { it back out }
         StringGrid1.Scrollbars := ssBoth;
       end;
     if tw<>cw then
       sw := sw + cw - tw - 1;
     ColWidths[4] := sw;
   end;
end;

procedure TForm1.SelectType;

var
  strTemp    : String[8];
  strName    : String[8];
  strExePath : string[120];
  Module     : TModuleEntry;
  FSearchrec : TSearchRec;
  FindResult : integer;
  f, i, w    : integer;

begin { TForm1.SelectTypeClick }
    { clear the grid }
  Screen.Cursor := crHourGlass;
  Application.Processmessages;
  with StringGrid1 do
   begin
    for i := 0 to 4 do
      Cols[i].BeginUpdate;
    Row := 1;
    Cells[0,1] := '';
    Cells[1,1] := '';
    Cells[2,1] := '';
    Cells[3,1] := '';
    Cells[4,1] := '';
   end;
  i := 1;
  w := 0;
  f := GetFtype;
  Module.dwSize := SizeOf(Module);

  if ModuleFirst(@Module) then
    repeat
      strName := StrPas(Module.szModule);
      strExePath := strpas(Module.szExepath);
      if length(strExePath)>w then
        w := length(strExePath);
      FindResult := FindFirst(strExePath, faAnyFile, FSearchRec);
      strTemp := Copy(strExepath, length(strExepath)-2,3);
      if (f=0) or (uppercase(strTemp) = ModTypes[f]) and (strName<>'') then
       with StringGrid1 do
        begin
          if i=RowCount then
            RowCount   := i+1;
          Cells[0,i] := ' '+strName;
          Cells[1,i] := FormatFloat('## ',Module.wUsageFlags);
          Cells[2,i] := FormatFloat('###,###,##0 ',FSearchrec.size);
          Cells[3,i] := FormatDateTime( 'm/dd/yyyy  hh:nn:ss ',
                                                     FileDateToDateTime(FSearchRec.Time));
          Objects[3,i] := pointer(FSearchRec.Time);
          Cells[4,i] := ' '+strExePath;
          Inc( i );
        end;
    until ModuleNext(@Module)=false;
  with StringGrid1 do
   begin
    if i>2 then
      RowCount := i
    else
      RowCount := 2;
     if i=1 then
       Cells[0,1] := '(none)';
   end;
  AdjustLastColWidth;
  SortGrid(GetColm);
  for i := 0 to 4 do
    StringGrid1.Cols[i].EndUpdate;
  Screen.Cursor := crDefault;
  Form1.Invalidate;
end; { TForm1.SelectType }

function Padstr(s: string;n: SmallInt): string;
var
  ts: string;
begin
  ts := s;
  while (length(ts)<n) do
   ts := ' '+ts;
  Result := ts;
end;

function CompareCellTxt(var SG: TStringGrid; x,y,m: integer): boolean; far;
begin
  Result := Padstr(SG.Cells[x,y],m) > Padstr(SG.Cells[x,y+1],m);
end;

function CompareLongInt(var SG: TStringGrid; x,y,m: integer): boolean; far;
begin
  Result := longint(SG.Objects[x,y]) > longint(SG.Objects[x,y+1]);
end;

procedure TForm1.SortGrid( const ColNum : integer );
var
  bSorted  : boolean;
  c,i,j,m  : integer;
  CFunc: CfType;
  sTmpVal  : string[120];
  sTmpPtr  : pointer;
begin { TForm1.SortGrid }
  SetColm(ColNum);
  bSorted := False;
  m := 0;
  with StringGrid1 do
   if (ColNum>0)and(ColNum<4) then
    for i := 1 to RowCount-2 do
     if length(Cells[ColNum,i])>m then
      m := length(Cells[ColNum,i]);
  if m>50 then m:=0;
  c := ColNum;
  if c=3 then
    CFunc := CompareLongint
  else
    CFunc := CompareCellTxt;

  while not bSorted do
   with StringGrid1 do
    begin
      bSorted := True;
      for i := 1 to RowCount-2 do
        begin
          if Cfunc(StringGrid1,c,i,m) then
            begin
              bSorted := False;
              for j := 0 to 4 do
               begin
                sTmpVal        := Cells[j,i];
                sTmpPtr        := Objects[j,i];
                Cells[j,i]     := Cells[j,i+1];
                Objects[j,i]   := Objects[j,i+1];
                Cells[j,i+1]   := sTmpVal;
                Objects[j,i+1] := sTmpPtr;
               end;
            end;
        end;
    end;
  StringGrid1.SetFocus;
end; { TForm1.SortGrid }

procedure TForm1.StringGrid1MouseDown(Sender: TObject; Button: TMouseButton;
                                        Shift: TShiftState; X, Y: Integer);
var
  c, r : longint;
begin

  if (Button = mbLeft) and (Shift = [ssLeft]) then
   with StringGrid1 do
    begin
      MouseToCell( X, Y, c, r );
      if (X>0)and(X<Width)and(Y>0)and(Y<StringGrid1.Height) then
       if (r = 0) and (GetColm<>c) then
        begin
          Screen.Cursor := crHourGlass;
          Application.Processmessages;
          for r := 0 to ColCount-1 do
            StringGrid1.Cols[r].BeginUpdate;
          SortGrid( c );
          for r := 0 to ColCount-1 do
            StringGrid1.Cols[r].EndUpdate;
          Screen.Cursor := crDefault;
        end;
    end;

end;

procedure TForm1.StringGrid1TopLeftChanged(Sender: TObject);
begin
 with StringGrid1 do
  if (Row<TopRow) or (Row>TopRow+VisibleRowCount) then
    begin
      Button2.Enabled := false;
      Button5.Enabled := false;
    end
  else
    begin
      Button2.Enabled := true;
      Button5.Enabled := true;
    end;
end;

procedure TForm1.StringGrid1SelectCell(Sender: TObject; Col, Row: Longint;
                                                        var CanSelect: Boolean);
begin
 with StringGrid1 do
  if (Cells[0,1]<>'(none)')and(pos('.DLL',Cells[4,Row])>0) then
    begin
      Button2.Enabled := true;
      Button4.Enabled := true;
      Button5.Enabled := true;
    end
  else
    begin
      Button2.Enabled := false;
      Button4.Enabled := false;
      Button5.Enabled := false;
    end;
end;

procedure TForm1.DecrementModuleUsage;
var
  strTemp    : String[12];
  Module     : TModuleEntry;
  ecode      : integer;
  Done       : boolean;
begin
 with StringGrid1 do
  begin
   strTemp := Cells[0,Row];
   while (strTemp[1]=' ')and(strTemp<>'') do
     delete(strTemp,1,1);
   fillchar(Module,sizeof(Module),0);
   Module.dwSize := Sizeof(Module);
   Done := false;

   if ModuleFirst(@Module) then
    while (not(Done) and (strTemp<>StrPas(Module.szModule))) do
     begin
      Application.Processmessages;
      Done := ModuleNext(@Module)=false;
     end;
   if (strTemp=StrPas(Module.szModule)) then
     FreeLibrary(Module.hModule);
   if ModuleFirst(@Module) then
    while (not(Done) and (strTemp<>StrPas(Module.szModule))) do
     begin
      Application.Processmessages;
      Done := ModuleNext(@Module)=false;
     end;

   if (strTemp=StrPas(Module.szModule)) then
     Cells[1,Row] := FormatFloat('## ', Module.wUsageFlags)
   else
     SelectType;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  OpenDialog: TOpenDialog;
  hHND: THandle;
  pFileName: array[0..120] of char;
begin
  OpenDialog:= TOpenDialog.Create(Self);
  with OpenDialog do
    begin
      Filter := 'DLL Files|*.DLL';
      DefaultExt := 'DLL';
      Options := [ofFileMustExist,ofPathMustExist];
      if Execute then
        begin
          StrPCopy(pFileName,Filename);
          hHND := LoadLibrary(pFileName);
          if hHND>HINSTANCE_ERROR then
            SelectType;
        end;
    end;
  OpenDialog.Free;
end;

procedure TForm1.Button2Click(Sender: TObject); { Increment Usage }
var
  strTemp : String[12];
  Module  : TModuleEntry;
  hHND: THandle;
  pFileName : array[0..120] of char;
  Done : boolean;
begin
 with StringGrid1 do
  begin
    Done := false;
    fillchar(Module,sizeof(Module),0);
    Module.dwSize := Sizeof(Module);
    strTemp := Cells[0,Row];
    StrPcopy(pFileName,Cells[4,Row]);
    hHND := LoadLibrary(pFileName);
    if hHND>HINSTANCE_ERROR then
      begin
        if ModuleFirst(@Module) then
         while (not(Done) and (strTemp<>StrPas(Module.szModule))) do
          begin
           Application.Processmessages;
           Done := ModuleNext(@Module)=false;
          end;
         Cells[1,Row] := FormatFloat('## ', Module.wUsageFlags)
      end;
  end;
end;

function  TForm1.FormattedOutput(var n: integer): string;
var
  w : integer;
  trec: string[120];
begin
 with StringGrid1 do
  begin
   w := 50 + length(Cells[4,n]);
   Result := PadStr('',w);
   TRec := Cells[0,n];                  { module name }
   move(TRec[2],Result[ 3],length(Trec)-1);{ skip leading space }
   TRec := PadStr(Cells[1,n],3);        { usage }
   move(TRec[1],Result[12],length(Trec));
   TRec := PadStr(Cells[2,n],10);       { file size }
   move(TRec[1],Result[15],length(Trec));
   TRec := PadStr(Cells[3,n],21);       { date & time }
   move(TRec[1],Result[26],length(Trec));
   TRec := Cells[4,n];                  { d:\path\filename }
   move(TRec[1],Result[47],length(Trec));
  end;
end;

function PrintedListHeading(const n:integer): string;
begin
  if n =1 then
    Result := '  MODULE  USAGE   SIZE      DATE       TIME        PATH & FILE NAME'
  else
    Result := '---------------------------------------------------------------------';
end;

procedure TForm1.Button3Click(Sender: TObject); { Print List }
const
  working : boolean = false;
var
  p, h, i: integer;
  cpage: integer;
begin
  if working then exit;
  working := true;
  Button3.Enabled := false;
  with Printer do
   begin
    p := 0;
    cpage := 0;
    h := abs(Canvas.Font.Height);
    Canvas.Brush.Style := bsClear;
    i := 1;
    BeginDoc;
    while (i<StringGrid1.RowCount)and(not(Printer.Aborted)) do
      begin
        if ((i mod 57)=1) then
          begin
            if (cpage>0) then
              p := PageHeight * cpage;
            Canvas.TextOut(0,p,PrintedListHeading(1));
            inc(p,h);
            Canvas.TextOut(0,p,PrintedListHeading(2));
            inc(p,h);
            inc(cpage);
          end;
        Canvas.TextOut(0,p,FormattedOutput(i));
        inc(p,h);
        inc(i);
      end;
    EndDoc;
   end;
  working := false;
  Button3.Enabled := true;
end;

procedure TForm1.Button4Click(Sender: TObject); { UnLoad DLL }
var
  strTemp : string[12];
  ModType : string[3];
begin
 with StringGrid1 do
  if (Cells[0,Row]<>'(none)') then
    begin
      strTemp := Cells[0,Row];
      ModType := copy(Cells[4,Row],length(Cells[4,Row])-2,3);
      if MessageDLG('Are you sure you want to '#13+
                    ' UnLoad - '+strTemp+'.'+ModType,
                                        mtConfirmation,[mbYes,mbNo],0) = mrYes then
        while (strTemp=Cells[0,Row]) do
          DecrementModuleUsage;
    end
  else
    MessageBeep(0);
end;

procedure TForm1.Button5Click(Sender: TObject); { Decrement Usage }
var
  strTemp    : String[12];
  ModType    : String[3];
begin
 with StringGrid1 do
  if (Cells[0,Row]<>'(none)') then
    begin
      strTemp := Cells[0,Row];
      ModType := copy(Cells[4,Row],length(Cells[4,Row])-2,3);
      if MessageDLG('Are you sure you want to Decrement'#13+
                    ' the Usage of Module - '+strTemp+'.'+ModType,
                                        mtConfirmation,[mbYes,mbNo],0) = mrYes then
        DecrementModuleUsage;
    end
  else
    MessageBeep(0);
end;

procedure TForm1.Button6Click(Sender: TObject); { Refresh List }
begin
  SelectType;
end;

procedure TForm1.ModuleClick(Sender: TObject);
var
  T: integer;
begin
  with (Sender as TMenuItem) do
   if GetFType<>Tag then
    begin
      All1.Checked := false;
      Exe1.Checked := false;
      Dll1.Checked := false;
      VBX1.Checked := false;
      DRV1.Checked := false;
      FON1.Checked := false;
      Checked := true;
      SetFType(Tag);
      SelectType;
    end;
end;

procedure TForm1.SaveList1Click(Sender: TObject);
var
  OpenDialog: TOpenDialog;
  SaveFile: TextFile;
  pRec: String;
  tRec: String[120];
  i, w: integer;
begin
 with StringGrid1 do
  if (Cells[0,Row]<>'(none)') then
   begin
    OpenDialog:= TOpenDialog.Create(Self);
    with OpenDialog do
      begin
        Title  := 'Save Module List';
        Options :=[ofHideReadOnly];
        Filter := 'Report File (*.RPT)|*.RPT|Text File (*.TXT)|*.TXT|All Files (*.*)|*.*';
        FileName := ChangeFileExt(paramstr(0),'.RPT');
        DefaultExt := 'RPT';
        if Execute then
          begin
            AssignFile(SaveFile,Filename);
            rewrite(SaveFile);
            writeln(SaveFile,PrintedListHeading(1));
            writeln(SaveFile,PrintedListHeading(2));
            for i := 1 to RowCount-1 do
              writeln(SaveFile,FormattedOutput(i));
            CloseFile(SaveFile);
          end;
      end;
    OpenDialog.Free;
   end
  else
   MessageBeep(0);
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
  AboutForm := TAboutForm.Create(Application);
  try
    AboutForm.ShowModal;
  finally
    AboutForm.Free;
  end;
end;

procedure TForm1.StringGrid1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (lo(Key)=VK_LEFT)
  or (lo(Key)=VK_RIGHT) then
   with StringGrid1 do
    begin
      if lo(Key)=VK_LEFT then      { Left Arrow }
        if LeftCol>0 then
          LeftCol := LeftCol -1
        else
      else                         { Right Arrow }
        if LeftCol+VisibleColCount<5 then
          LeftCol := LeftCol +1;
      Key := 0;
    end;
end;

procedure TForm1.StringGrid1DrawCell(Sender: TObject; Col, Row: Longint;
  Rect: TRect; State: TGridDrawState);
var
  tw: integer;
begin
   if ((Col=1)or(Col=2)or(Col=3)) then
    with (Sender as TStringGrid).Canvas do   { Right Justify routine }
     begin
       tw := Rect.Right - TextWidth(StringGrid1.Cells[Col,Row]) - 2;
       if tw<Rect.Left+2 then tw := Rect.Left+2;
       TextRect(Rect, tw, Rect.Top+2, StringGrid1.Cells[Col,Row]);
    end;
end;

end.


